﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Threading.Tasks;
using System.Windows.Forms;

using OWLIFT;

namespace OWLIFTViewCS
{
    public partial class Form1 : Form
    {
        private OwDev m_owDev;
        private byte[] m_decodeBuf;
        private ushort[] m_tempTab;
        private Bitmap m_prevBmp;
        private Object m_lock = new Object();
        private Timer m_timer = new Timer();
        private UInt16 m_fpaTemp;
        private UInt16 m_housingTemp;
        private UInt32 m_frameCounter;
        private Font m_font;

        public Form1()
        {
            InitializeComponent();
        }

        private void buttonStart_Click(object sender, EventArgs e)
        {
            try
            {
                OwDev[] owDevs = OwDev.GetDevices();

                m_owDev = owDevs[0];
                buttonStart.Enabled = true;
                buttonStop.Enabled = false;

                m_owDev.CaptureSetup(OwFrameRate.F8P6, CaptureProc);
                m_owDev.CaptureStart();

                buttonStart.Enabled = false;
                buttonStop.Enabled = true;

                m_decodeBuf = new byte[m_owDev.FrameWidth * m_owDev.FrameHeight * 3];
                m_tempTab = new ushort[m_owDev.FrameWidth * m_owDev.FrameHeight];
                m_prevBmp = new Bitmap(picBoxPreview.Width, picBoxPreview.Height);

                m_timer.Interval = 1000;
                m_timer.Tick += new EventHandler(Timer_Tick);
                m_timer.Start();

                m_font = new Font("Arial", 14);
            }
            catch (OwException ex)
            {
                buttonStart.Enabled = false;
                buttonStop.Enabled = false;
                buttonRunFFC.Enabled = false;
                buttonSysFFCMode.Enabled = false;
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK);
            }
        }

        private void Release()
        {
            if (m_owDev != null)
            {
                m_owDev.CaptureStop();
                m_owDev = null;
            }
        }

      　private void buttonStop_Click(object sender, EventArgs e)
        {
            Release();
            m_decodeBuf = null;
            m_tempTab = null;
            m_prevBmp = null;
            buttonStart.Enabled = true;
            buttonStop.Enabled = false;
        }

        private void buttonSysFFCMode_Click(object sender, EventArgs e)
        {
            UInt16[] data = new UInt16[16];

            m_owDev.CommandGet(0x0200, 0x3C, data);
            string msg = "";

            for (int i = 0; i < data.Length; ++i)
            {
                msg += data[i].ToString("X4") + " ";
                if (i % 4 == 3)
                    msg += "\n";
            }

            MessageBox.Show(msg, "FFC Mode", MessageBoxButtons.OK);
        }

        private void buttonRunFFC_Click(object sender, EventArgs e)
        {
            try
            {
                m_owDev.CommandRun(0x0200, 0x40);
            }
            catch (OwException ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK);
            }
        }

        private void SetLabelTemp(Label label, UInt16 temp)
        {
            label.Text = (((double)temp / 100)).ToString("F2");
        }

        private void SetLabelUInt32(Label label, UInt32 val)
        {
            label.Text = val.ToString();
        }

        delegate void UpdateInfoLabelsDelegate(UInt16 fpaTemp, UInt16 housingTemp, UInt32 frameCounter);

        private void UpdateInfoLabels(UInt16 fpaTemp, UInt16 housingTemp, UInt32 frameCounter)
        {
            try
            {
                SetLabelTemp(labelFPATemp, fpaTemp);
                SetLabelTemp(labelHousingTemp, housingTemp);
                SetLabelUInt32(labelFrameCounter, frameCounter);
            }
            catch (System.ObjectDisposedException)
            {
                // ignore
            }
        }

        private unsafe void CaptureProc(byte[] buf)
        {
            int fx = m_owDev.FrameWidth, fy = m_owDev.FrameHeight;
            Bitmap bmp;

            try
            {
                m_owDev.Decode(buf, m_decodeBuf, OwDecodeType.AGC_BLUE_ORANGE_24);
                m_owDev.GetTempTable(m_tempTab);
            }
            catch (OwException)
            {
                return;
            }

            m_owDev.FinishDecode();

            fixed (void* p = m_decodeBuf)
            {
                bmp = new Bitmap(fx, fy, fx * 3,
                    PixelFormat.Format24bppRgb, new IntPtr(p));
            }

            int x = fx / 2;
            int y = fy / 2;
            double temp = (double)m_tempTab[y * fx + x] / m_owDev.LastTempResolution - 273.15;
            string temps = temp.ToString("F2") + "C";
            
            lock (m_lock)
            {
                Graphics g = Graphics.FromImage(m_prevBmp);
                int px = picBoxPreview.Width;
                int py = picBoxPreview.Height;

                g.DrawImage(bmp, 0, 0, px, py);
                g.DrawLine(Pens.White, px / 2 - 2, py / 2, px / 2 + 2, py / 2);
                g.DrawLine(Pens.White, px / 2, py / 2 - 2, px / 2, py / 2 + 2);
                g.DrawString(temps, m_font, Brushes.White, px / 2 + 1, py / 2 + 1);

                picBoxPreview.Image = m_prevBmp;

                // Telemetry Data
                LepTelemetryDataLineA telem = new LepTelemetryDataLineA(m_owDev, buf);
                m_fpaTemp = telem.FPATemp_Kx100;
                m_housingTemp = telem.HousingTemp_Kx100;
                m_frameCounter = telem.FrameCounter;
            }
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            Task.Factory.StartNew(() =>
            {
                try
                {
                    Invoke(new UpdateInfoLabelsDelegate(UpdateInfoLabels),
                        new object[] { m_fpaTemp, m_housingTemp, m_frameCounter });
                }
                catch (System.ObjectDisposedException)
                {
                    // ignore
                }
            });
        }

        private void buttonExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_timer.Stop();
            Release();
        }

        private void Form1_SizeChanged(object sender, EventArgs e)
        {
            Rectangle rForm = this.ClientRectangle;
            Rectangle rPic = picBoxPreview.Bounds;

            if (rForm.Width == 0)
                return;

            rPic.Width = rForm.Right - rPic.Left - 12;
            rPic.Height = rForm.Bottom - rPic.Top - 12;

            lock (m_lock)
            {
                picBoxPreview.Bounds = rPic;
                m_prevBmp = new Bitmap(picBoxPreview.Width, picBoxPreview.Height);
            }
        }
    }
}
